package benefactorservice

import (
	"context"
	"git.gocasts.ir/ebhomengo/niki/entity"
	benefactoreparam "git.gocasts.ir/ebhomengo/niki/param/benefactor/benefactore"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
)

func (s Service) LoginOrRegister(ctx context.Context, req benefactoreparam.LoginOrRegisterRequest) (benefactoreparam.LoginOrRegisterResponse, error) {
	const op = "benefactorservice.LoginOrRegister"

	code, gErr := s.redisOtp.GetCodeByPhoneNumber(ctx, req.PhoneNumber)
	if gErr != nil {
		return benefactoreparam.LoginOrRegisterResponse{}, richerror.New(op).WithErr(gErr).WithKind(richerror.KindUnexpected)
	}
	if code == "" || code != req.VerificationCode {
		return benefactoreparam.LoginOrRegisterResponse{}, richerror.New(op).WithMessage(errmsg.ErrorMsgOtpCodeIsNotValid).WithKind(richerror.KindForbidden)
	}

	deleted, dErr := s.redisOtp.DeleteCodeByPhoneNumber(ctx, req.PhoneNumber)
	if dErr != nil {
		return benefactoreparam.LoginOrRegisterResponse{}, richerror.New(op).WithErr(gErr).WithKind(richerror.KindUnexpected)
	}
	if !deleted {
		//TODO - add to log
	}

	isExist, benefactor, rErr := s.repo.IsExistBenefactorByPhoneNumber(ctx, req.PhoneNumber)
	if rErr != nil {
		return benefactoreparam.LoginOrRegisterResponse{}, richerror.New(op).WithErr(rErr).WithKind(richerror.KindUnexpected)
	}
	if !isExist {
		newBenefactor, err := s.repo.CreateBenefactor(ctx, entity.Benefactor{
			PhoneNumber: "",
			Status:      entity.BenefactorActiveStatus,
			Role:        entity.UserBenefactorRole,
		})
		if err != nil {
			return benefactoreparam.LoginOrRegisterResponse{}, richerror.New(op).WithErr(rErr).WithKind(richerror.KindUnexpected)
		}

		benefactor = newBenefactor
	}

	accessToken, err := s.auth.CreateAccessToken(benefactor)
	if err != nil {
		return benefactoreparam.LoginOrRegisterResponse{}, richerror.New(op).WithErr(rErr).WithKind(richerror.KindUnexpected)
	}

	refreshToken, err := s.auth.CreateRefreshToken(benefactor)
	if err != nil {
		return benefactoreparam.LoginOrRegisterResponse{}, richerror.New(op).WithErr(rErr).WithKind(richerror.KindUnexpected)
	}

	return benefactoreparam.LoginOrRegisterResponse{
		BenefactorInfo: benefactoreparam.BenefactroInfo{
			ID:        benefactor.ID,
			FirstName: benefactor.FirstName,
			LastName:  benefactor.LastName,
			Role:      benefactor.Role.String(),
		},
		Tokens: benefactoreparam.Tokens{
			AccessToken:  accessToken,
			RefreshToken: refreshToken,
		},
	}, nil
}
